/*
 * $Copyright Open Broadcom Corporation$
 *
 * $Id: linux_igd.c,v 1.10.2.2 2003/10/31 21:31:36 mthawani Exp $
 */

#include <sys/ioctl.h>
#include <net/if.h>
//#include <linux/sockios.h>
#include <linux/socket.h>
#include <signal.h>
#include <signal.h>

#define __KERNEL__
#include <asm/types.h>
#include <linux/sockios.h>
#include <linux/ethtool.h>

#include <string.h>         // for memset, strncasecmp, etc.
#include <stdio.h>          // for printf, perror, fopen, fclose, etc.
#include <arpa/inet.h>      // for inet_aton, etc.
#include <time.h>           // for time, strftime, gmtime, etc.
#include <stdlib.h>         // for malloc, free, atoi, atol, etc.

#include "upnp_dbg.h"
#include "upnp_osl.h"
#include "upnp.h"
#include "../igd/igd.h"
#include "shutils.h"
#include "bcmnvram.h"

#include "syscall.h"
#include "ifcdefs.h"

#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <bcmtypes.h>
#include "bcmatmapi.h"
#include "atmapidrv.h"
#include "bcmadsl.h"
#include "adsldrv.h"
#include "board_api.h"


#define _PATH_PROCNET_DEV           "/proc/net/dev"
#define CMD_BUF_SZ                  256
#define IFC_STATUS_SZ               16
#define IFC_PATH_SZ                 256

extern int bcmSystemEx ( char *cmd, int printFlag );

int g_AtmDevFd  = -1;
int g_AdslDevFd = -1;

static int openAdslDevice ( void )
{
    int nFd = -1;

    nFd = open( "/dev/bcmadsl0", O_RDWR );

    if( nFd == -1 ) {
        printf( "openAdslDevice: open error %d\n", errno );
    }

    UPNP_TRACE ( ("openAdslDevice nFd %d\n", nFd) );

    return( nFd );
}

static int openAtmDevice ( void )
{
    int nFd = -1;

    nFd = open( "/dev/bcmatm0", O_RDWR );

    if( nFd == -1 ) {
        printf( "openAtmDevice: open error %d\n", errno );
    }

    UPNP_TRACE ( ("openAtmDevice nFd %d\n", nFd) );

    return( nFd );
}

static BCMATM_STATUS getInterfaceId ( UINT8 ucPhyPort,
                                      UINT32 *pulInterfaceId )
{
    ATMDRV_INTERFACE_ID Arg = {ucPhyPort,{0,0,0}, 0, STS_ERROR};

    if ( g_AtmDevFd == -1 ) {
        g_AtmDevFd = openAtmDevice();
    }

    if( g_AtmDevFd != -1  )
    {
        Arg.ulInterfaceId = *pulInterfaceId;
        ioctl( g_AtmDevFd, ATMIOCTL_GET_INTERFACE_ID, &Arg );
        *pulInterfaceId = Arg.ulInterfaceId;
    } else {
        Arg.baStatus = STS_STATE_ERROR;
    }

    return( Arg.baStatus );

}

static BCMATM_STATUS getInterfaceCfg ( UINT32 ulInterfaceId,
                                       PATM_INTERFACE_CFG pInterfaceCfg )
{
    ATMDRV_INTERFACE_CFG Arg = {ulInterfaceId, pInterfaceCfg, STS_ERROR};

    if ( g_AtmDevFd == -1 ) {
        g_AtmDevFd = openAtmDevice();
    }

    if( g_AtmDevFd != -1  ) {
        ioctl( g_AtmDevFd, ATMIOCTL_GET_INTERFACE_CFG, &Arg );
    } else {
        Arg.baStatus = STS_STATE_ERROR;
    }

    return( Arg.baStatus );

}

static BCMATM_STATUS getInterfaceStatistics ( UINT32 ulInterfaceId,
                                              PATM_INTERFACE_STATS pStatistics,
                                              UINT32 ulReset )
{
    ATMDRV_INTERFACE_STATISTICS Arg = {ulInterfaceId, pStatistics, ulReset,
        STS_ERROR};

    if ( g_AtmDevFd == -1 ) {
        g_AtmDevFd = openAtmDevice();
    }

    if( g_AtmDevFd != -1  ) {
        ioctl( g_AtmDevFd, ATMIOCTL_GET_INTERFACE_STATISTICS, &Arg );
    } else {
        Arg.baStatus = STS_STATE_ERROR;
    }

    return( Arg.baStatus );
}

int osl_ifstats(char *ifname, if_stats_t *pstats )
{
    int err = -1;
    memset(pstats, 0, sizeof(*pstats));
    ATM_INTERFACE_CFG Cfg;
    ATM_INTERFACE_STATS Stats;
    PATM_INTF_AAL5_AAL0_STATS pAal5Stats = &Stats.Aal5IntfStats;
    UINT32 ulInterfaceId = 0;
    UINT32 ulReset = 0;
    long i;

    for( i = 0; i < PHY_NUM_PORTS; i++ ) {
        ulInterfaceId = (UINT32) i;
        getInterfaceId( (UINT8) i, &ulInterfaceId );
        Cfg.ulStructureId = ID_ATM_INTERFACE_CFG;
        Stats.ulStructureId = ID_ATM_INTERFACE_STATS;

        if( getInterfaceCfg( ulInterfaceId, &Cfg ) == STS_SUCCESS &&
            Cfg.ulIfOperStatus == OPRSTS_UP &&
            getInterfaceStatistics(ulInterfaceId, &Stats,
                ulReset) == STS_SUCCESS )
        {
            err = 0;
            if( pAal5Stats->ulIfInOctets || pAal5Stats->ulIfOutOctets )
            {
                pstats->tx_packets = pAal5Stats->ulIfOutUcastPkts;
                pstats->rx_packets = pAal5Stats->ulIfInUcastPkts;
                pstats->tx_bytes   = pAal5Stats->ulIfOutOctets;
                pstats->rx_bytes   = pAal5Stats->ulIfInOctets;
                pstats->rx_dropped = pAal5Stats->ulIfInDiscards;
                pstats->tx_dropped = pAal5Stats->ulIfOutDiscards;
                pstats->rx_errors  = pAal5Stats->ulIfInErrors;
                pstats->tx_errors  = pAal5Stats->ulIfOutErrors;
            }
        }
    }

    return err;
}

void osl_igd_enable ()
{
    ADSLDRV_STATUS_ONLY Arg;

    Arg.bvStatus = BCMADSL_STATUS_ERROR;

    if( g_AdslDevFd == -1  )
        g_AdslDevFd = openAdslDevice();

    if( g_AdslDevFd != -1  )
        ioctl( g_AdslDevFd, ADSLIOCTL_CONNECTION_START, &Arg );
    else
        Arg.bvStatus = BCMADSL_STATUS_ERROR;

    return;
} /* BcmAdsl_ConnectionStart */


void osl_igd_disable()
{
    ADSLDRV_STATUS_ONLY Arg;

    Arg.bvStatus = BCMADSL_STATUS_ERROR;

    if( g_AdslDevFd == -1  )
        g_AdslDevFd = openAdslDevice();

    if( g_AdslDevFd != -1  )
        ioctl( g_AdslDevFd, ADSLIOCTL_CONNECTION_STOP, &Arg );
    else
        Arg.bvStatus = BCMADSL_STATUS_ERROR;

    return;
} /* BcmAdsl_ConnectionStop */


/* Return OSL_LINK_UP if the link status given interface is UP, OSL_LINK_DOWN otherwise. */
osl_link_t osl_link_status(char *devname)
{
    uint if_up = OSL_LINK_DOWN;
    ADSLDRV_CONNECTION_INFO Arg;

    if ( g_AdslDevFd == -1 ) {
        g_AdslDevFd = openAdslDevice ();
    }
    if ( g_AdslDevFd != -1 ) {
        ioctl ( g_AdslDevFd, ADSLIOCTL_GET_CONNECTION_INFO, &Arg );
        if ( Arg.ConnectionInfo.LinkState == BCM_ADSL_LINK_UP ) {
            if_up = OSL_LINK_UP;
        }
    }

    return if_up;
}


uint osl_max_bitrates(char *devname, ulong *rx, ulong *tx)
{
    ADSLDRV_CONNECTION_INFO Arg;
    Arg.bvStatus = BCMADSL_STATUS_ERROR;

    *rx = *tx = 0;

    if ( g_AdslDevFd == -1 ) {
        g_AdslDevFd = openAdslDevice ();
    }
    if ( g_AdslDevFd != -1 ) {
        ioctl ( g_AdslDevFd, ADSLIOCTL_GET_CONNECTION_INFO, &Arg );
        if ( Arg.ConnectionInfo.LinkState == BCM_ADSL_LINK_UP ) {
            if ( Arg.ConnectionInfo.ulInterleavedDnStreamRate != 0 ) {
                *rx = Arg.ConnectionInfo.ulInterleavedDnStreamRate;
                *tx = Arg.ConnectionInfo.ulInterleavedUpStreamRate;
            } else {
                *rx = Arg.ConnectionInfo.ulFastDnStreamRate;
                *tx = Arg.ConnectionInfo.ulFastUpStreamRate;
            }
        }
    } else {
        return FALSE;
    }

    return TRUE;
}


void osl_sys_restart()
{
    kill(1, SIGHUP);
}

void osl_sys_reboot()
{
    kill(1, SIGTERM);
}

bool read_if_status ( char *path )
{
    bool status = FALSE;
    char data[IFC_STATUS_SZ];
    FILE *fs = fopen ( path, "r" );

    if ( fs == NULL ) {
        return status;
    }

    data[0] = '\0';
    fgets (data, IFC_STATUS_SZ, fs );
    fclose ( fs );

    if ( data != NULL ) {
        int i = 0;
        while ( ( data[i] != '\0')           &&
                ( isspace ( data[i] ) == 0 ) &&
                ( i < IFC_STATUS_SZ ) ) {
            i++;
            data[i] = '\0';
        }
    }
    if ( atoi ( data ) == IFC_STATUS_UP ) {
        status = TRUE;
    }
    return status;
}

bool osl_wan_isup(char *devname)
{
    char tmp[100];
    char ifc_path[IFC_PATH_SZ];
    bool status = FALSE;
    if ( strcasecmp ( nvram_safe_get ( igd_pri_wan_var ( tmp, sizeof ( tmp ),
                                       "proto" ) ), "disabled" ) != 0 ) {
        if (  strncasecmp ( devname, "ppp_", strlen ( "ppp_" ) ) == 0 ) {
            snprintf ( ifc_path, IFC_PATH_SZ, "/proc/var/fyi/wan/%s/status", devname );
            return ( read_if_status ( ifc_path ) );
        } else {
            return ( status = osl_link_status ( "/dev/bcmadsl0" ) );
        }
    } else {
        return status;
    }
}

void osl_igd_ppp_conn_disable ( char *wanif )
{
    char cmd[CMD_BUF_SZ];

    if ( wanif == NULL ) {
        return;
    }

    snprintf ( cmd, CMD_BUF_SZ, "echo 0 > /proc/var/fyi/wan/%s/wanup", wanif );
    bcmSystemEx( cmd, 1 );
    sysWakeupMonitorTask();
    return;
}

void osl_igd_ppp_conn_enable ( char *wanif )
{
    char cmd[CMD_BUF_SZ];

    if ( wanif == NULL ) {
        return;
    }

    snprintf ( cmd, CMD_BUF_SZ, "echo 1 > /proc/var/fyi/wan/%s/wanup", wanif );
    bcmSystemEx( cmd, 1 );
    sysWakeupMonitorTask();
    return;
}

