/*
 * $Copyright Open Broadcom Corporation$
 *
 * $Id: wanpotslinkconfig.c,v 1.4.20.2 2003/10/31 21:31:35 mthawani Exp $
 */

#include "upnp_dbg.h"
#include "upnp_osl.h"
#include "upnp.h"
#include "wandsllinkconfig.h"

#include "netconf.h"
#include "bcmnvram.h"
#include "nvparse.h"
#include "mapmgr.h"

#define GetModulationType		NotImplemented
#define SetDestinationAddress		NotImplemented
#define GetDestinationAddress		NotImplemented
#define SetATMEncapsulation		NotImplemented
#define GetATMEncapsulation		NotImplemented
#define GetFCSPreserved			NotImplemented
#define SetFCSPreserved			NotImplemented

#define GetDSLLinkInfo			DefaultAction
#define GetAutoConfig			DefaultAction

struct iftab {
    char *name;
    int  if_id;
};

struct iftab iftable[] = {
    { "nas_", 1 },
    { "ipa_", 2 },
    { "ppp_", 3 },
    { "ppp_", 4 },
    { "nas_", 5 },
    { "unconfigured", 6 },
    { NULL, 0 }
};

static char *PossibleLinkTypes_allowedValueList[] = { "EoA",
                                                      "IPoA",
                                                      "PPPoA",
						      "PPPoE",
						      "CIP",
						      "Unconfigured",
                                                      NULL };

static char *LinkStatus_allowedValueList[] = { "Down",
                                               "Up",
                                               "Initializing",
                                               "Unavailable",
                                               NULL };

static VarTemplate StateVariables[] = { 
    { "LinkType", "Unconfigured", VAR_STRING, 
      (allowedValue) { PossibleLinkTypes_allowedValueList } }, 
    { "LinkStatus", "Unavailable", VAR_EVENTED|VAR_STRING,
      (allowedValue) { LinkStatus_allowedValueList } }, 
    { "AutoConfig", "1", VAR_EVENTED|VAR_BOOL }, 
    { "ModulationType", "", VAR_STRING }, 
    { "DestinationAddress", "", VAR_STRING }, 
    { "ATMEncapsulation", "", VAR_STRING }, 
    { "FCSPreserved", "", VAR_BOOL }, 
    { 0 } 
};

#define VAR_LinkType		0
#define VAR_LinkStatus		1
#define VAR_AutoConfig		2
#define VAR_ModulationType	3
#define VAR_DestinationAddress	4
#define VAR_ATMEncapsulation	5
#define VAR_FCSPreserved	6


static int SetDSLLinkType(UFILE *uclient, PService psvc, PAction ac, pvar_entry_t args, int nargs);
static int WANDSLLinkConfig_GetVar(struct Service *psvc, int varindex);

static Action _SetDSLLinkType = { 
    "SetDSLLinkType", SetDSLLinkType,
    (Param [])    {
            {"NewLinkType", VAR_LinkType, VAR_IN},
            { 0 }
        }
};

static Action _GetDSLLinkInfo = { 
    "GetDSLLinkInfo", GetDSLLinkInfo,
    (Param [])    {
            {"NewLinkType", VAR_LinkType, VAR_OUT},
            {"NewLinkStatus", VAR_LinkStatus, VAR_OUT},
            { 0 }
        }
};

static Action _GetAutoConfig = { 
    "GetAutoConfig", GetAutoConfig,
    (Param [])    {
            {"NewAutoConfig", VAR_AutoConfig, VAR_OUT},
            { 0 }
        }
};

static Action _GetModulationType = { 
    "GetModulationType", GetModulationType,
    (Param [])    {
            {"NewModulationType", VAR_ModulationType, VAR_OUT},
            { 0 }
        }
};

static Action _SetDestinationAddress = { 
    "SetDestinationAddress", SetDestinationAddress,
    (Param [])    {
            {"NewDestinationAddress", VAR_DestinationAddress, VAR_IN},
            { 0 }
        }
};

static Action _GetDestinationAddress = { 
    "GetDestinationAddress", GetDestinationAddress,
    (Param [])    {
            {"NewDestinationAddress", VAR_DestinationAddress, VAR_OUT},
            { 0 }
        }
};

static Action _SetATMEncapsulation = { 
    "SetATMEncapsulation", SetATMEncapsulation,
    (Param [])    {
            {"NewATMEncapsulation", VAR_ATMEncapsulation, VAR_IN},
            { 0 }
        }
};

static Action _GetATMEncapsulation = { 
    "GetATMEncapsulation", GetATMEncapsulation,
    (Param [])    {
            {"NewATMEncapsulation", VAR_ATMEncapsulation, VAR_OUT},
            { 0 }
        }
};

static Action _SetFCSPreserved = { 
    "SetFCSPreserved", SetFCSPreserved,
    (Param [])    {
            {"NewFCSPreserved", VAR_FCSPreserved, VAR_IN},
            { 0 }
        }
};

static Action _GetFCSPreserved = { 
    "GetFCSPreserved", GetFCSPreserved,
    (Param [])    {
            {"NewFCSPreserved", VAR_FCSPreserved, VAR_OUT},
            { 0 }
        }
};

static PAction Actions[] = {
    &_SetDSLLinkType,
    &_GetDSLLinkInfo,
    &_GetAutoConfig,
    &_GetModulationType,
    &_SetDestinationAddress,
    &_GetDestinationAddress,
    &_SetATMEncapsulation,
    &_GetATMEncapsulation,
    &_SetFCSPreserved,
    &_GetFCSPreserved,
    NULL
};


ServiceTemplate Template_WANDSLLinkConfig = {
    "WANDSLLinkConfig:1",
    NULL,
    WANDSLLinkConfig_GetVar,	/* state variable handler */
    NULL,  /* SVCXML */
    ARRAYSIZE(StateVariables)-1, StateVariables,
    Actions
};

static int ifname_to_id ( char *ifname )
{
    int id = 0;

    for ( ; (iftable[id].name != NULL); id++ ) {
        if ( strncasecmp ( iftable[id].name, ifname,
             strlen ( iftable[id].name ) ) == 0 ) {
            return ( iftable[id].if_id );
        }
    }
    return ( id - 1 );
}

static int SetDSLLinkType(UFILE *uclient, PService psvc, PAction ac, pvar_entry_t args, int nargs)
{
    snprintf(psvc->vars[VAR_LinkType].value, sizeof(psvc->vars[VAR_LinkType].value), ac->params[0].value);

    return TRUE;
}

static int WANDSLLinkConfig_GetVar(struct Service *psvc, int varindex)
{
    PWANDSLLinkConfigData data = psvc->opaque;
    struct StateVar *var;
    char *cur;
    int if_id;

    var = &(psvc->vars[varindex]);
    switch (varindex) {
    case VAR_LinkType:
	cur = nvram_safe_get ( "wanif_name" );
	if_id = ifname_to_id ( cur );
        sprintf(var->value, "%s", PossibleLinkTypes_allowedValueList[if_id]);
        break;

    case VAR_LinkStatus:
	data->link_status = osl_link_status ( "/dev/bcmadsl0" );
        sprintf(var->value, "%s", LinkStatus_allowedValueList[data->link_status]);
        break;

    case VAR_AutoConfig:
        sprintf(var->value, "%d", 1);
        break;

    } /* end-switch */

    return TRUE;
}

