/*
 * $Copyright Open Broadcom Corporation$
 *
 * $Id: wanppp.c,v 1.33.2.2 2003/10/31 21:31:35 mthawani Exp $
 */

#include "upnp_dbg.h"
#include "upnp_osl.h"
#include "upnp.h"
#include "igd.h"
#include "wanppp.h"
#include "netconf.h"

#include "bcmnvram.h"
#include "nvparse.h"
#include "mapmgr.h"


/*
  Code to support the WAN IPConnection service.
*/

extern int GetGenericPortMappingEntry(UFILE *, PService , PAction, pvar_entry_t , int );
extern int GetSpecificPortMappingEntry(UFILE *, PService , PAction, pvar_entry_t , int);
extern int AddPortMapping(UFILE *, PService, PAction, pvar_entry_t, int);
extern int DeletePortMapping(UFILE *, PService, PAction, pvar_entry_t, int);
static int RequestConnection(UFILE *, PService , PAction, pvar_entry_t , int);
static int ForceTermination(UFILE *, PService, PAction, pvar_entry_t , int);
static int SetConnectionType(UFILE *uclient, PService psvc, PAction ac, pvar_entry_t args, int nargs);

extern int igd_config_generation;
extern char * safe_snprintf(char *str, int *len, const char *fmt, ...);
extern int nvram_commit ( void );
extern void osl_igd_ppp_conn_disable ( char *wanif );
extern void osl_igd_ppp_conn_enable ( char *wanif );

static int WANPPPConnectionInit(struct Service *psvc, service_state_t state);
static int WANPPPConnection_GetVar(struct Service *psvc, int varindex);
static void WANPPPConnection_Update(timer_t t, PService psvc);

bool set_auto_dns(char *AutoDNSEnable, char *ExternalDNSServers);

#define GetConnectionTypeInfo		DefaultAction
#define GetStatusInfo			DefaultAction
#define GetNATRSIPStatus		DefaultAction
#define GetExternalIPAddress		DefaultAction
#define GetIdleDisconnectTime		DefaultAction
#define GetWarnDisconnectDelay		DefaultAction
#define GetAutoDisconnectTime		DefaultAction

static char *PossibleConnectionTypes_allowedValueList[] = { "Unconfigured",
                                                            "IP_Routed",
                                                            "DHCP_Spoofed",
							    "PPPoE_Bridged",
							    "PPTP_Relay",
							    "L2TP_Relay",
							    "PPPoE_Relay",
                                                            NULL };

static char *ConnectionStatus_allowedValueList[] = { "Unconfigured",
                                                     "Connecting",
                                                     "Authenticating",
                                                     "Connected",
                                                     "PendingDisconnect",
                                                     "Disconnecting",
                                                     "Disconnected",
                                                     NULL };
static char *LastConnectionError_allowedValueList[] = { "ERROR_NONE", "ERROR_UNKNOWN", NULL };
static char *PortMappingProtocol_allowedValueList[] = { "TCP", "UDP", NULL };

static VarTemplate StateVariables[] = {
    { "ConnectionType", "IP_Routed", VAR_STRING|VAR_LIST,
      (allowedValue) { PossibleConnectionTypes_allowedValueList }  },
    { "PossibleConnectionTypes", "", VAR_EVENTED|VAR_STRING|VAR_LIST,
      (allowedValue) { PossibleConnectionTypes_allowedValueList } },
    { "ConnectionStatus", "", VAR_EVENTED|VAR_STRING|VAR_LIST,
      (allowedValue) { ConnectionStatus_allowedValueList } },
    { "Uptime", "", VAR_ULONG },
    { "UpstreamMaxBitRate", "", VAR_ULONG },
    { "DownstreamMaxBitRate", "", VAR_ULONG },
    { "LastConnectionError", "", VAR_STRING|VAR_LIST,
      (allowedValue) { LastConnectionError_allowedValueList } },
    { "RSIPAvailable", "1", VAR_BOOL },
    { "NATEnabled", "1", VAR_BOOL },
    { "ExternalIPAddress", "", VAR_EVENTED|VAR_STRING },
    { "PortMappingNumberOfEntries", "", VAR_EVENTED|VAR_USHORT },
    { "PortMappingEnabled", "", VAR_BOOL },
    { "PortMappingLeaseDuration", "", VAR_ULONG },
    { "RemoteHost", "", VAR_STRING },
    { "ExternalPort", "", VAR_USHORT },
    { "InternalPort", "", VAR_USHORT },
    { "PortMappingProtocol", "", VAR_STRING|VAR_LIST,
      (allowedValue) { PortMappingProtocol_allowedValueList } },
    { "InternalClient", "", VAR_STRING },
    { "PortMappingDescription", "", VAR_STRING },
    { NULL }
};

static Action _SetConnectionType = {
    "SetConnectionType", SetConnectionType,
   (Param []) {
       {"NewConnectionType", VAR_ConnectionType, VAR_IN},
       { 0 }
    }
};

static Action _GetConnectionTypeInfo = {
    "GetConnectionTypeInfo", GetConnectionTypeInfo,
   (Param []) {
       {"NewConnectionType", VAR_ConnectionType, VAR_OUT},
       {"NewPossibleConnectionTypes", VAR_PossibleConnectionTypes, VAR_OUT},
       { 0 }
    }
};

static Action _RequestConnection = {
    "RequestConnection", RequestConnection,
   (Param []) {
       { 0 }
    }
};

static Action _ForceTermination = {
    "ForceTermination", ForceTermination,
   (Param []) {
       { 0 }
    }
};

static Action _GetStatusInfo = {
    "GetStatusInfo", GetStatusInfo,
   (Param []) {
       {"NewConnectionStatus", VAR_ConnectionStatus, VAR_OUT},
       {"NewLastConnectionError", VAR_LastConnectionError, VAR_OUT},
       {"NewUptime", VAR_Uptime, VAR_OUT},
       { 0 }
    }
};

static Action _GetNATRSIPStatus = {
    "GetNATRSIPStatus", GetNATRSIPStatus,
   (Param []) {
       {"NewRSIPAvailable", VAR_RSIPAvailable, VAR_OUT},
       {"NewNATEnabled", VAR_NATEnabled, VAR_OUT},
       { 0 }
    }
};

static Action _GetGenericPortMappingEntry = {
    "GetGenericPortMappingEntry", GetGenericPortMappingEntry,
   (Param []) {
       {"NewPortMappingIndex", VAR_PortMappingNumberOfEntries, VAR_IN},
       {"NewRemoteHost", VAR_RemoteHost, VAR_OUT},
       {"NewExternalPort", VAR_ExternalPort, VAR_OUT},
       {"NewProtocol", VAR_PortMappingProtocol, VAR_OUT},
       {"NewInternalPort", VAR_InternalPort, VAR_OUT},
       {"NewInternalClient", VAR_InternalClient, VAR_OUT},
       {"NewEnabled", VAR_PortMappingEnabled, VAR_OUT},
       {"NewPortMappingDescription", VAR_PortMappingDescription, VAR_OUT},
       {"NewLeaseDuration", VAR_PortMappingLeaseDuration, VAR_OUT},
       { 0 }
    }
};

static Action _GetSpecificPortMappingEntry = {
    "GetSpecificPortMappingEntry", GetSpecificPortMappingEntry,
   (Param []) {
       {"NewRemoteHost", VAR_RemoteHost, VAR_IN},
       {"NewExternalPort", VAR_ExternalPort, VAR_IN},
       {"NewProtocol", VAR_PortMappingProtocol, VAR_IN},
       {"NewInternalPort", VAR_InternalPort, VAR_OUT},
       {"NewInternalClient", VAR_InternalClient, VAR_OUT},
       {"NewEnabled", VAR_PortMappingEnabled, VAR_OUT},
       {"NewPortMappingDescription", VAR_PortMappingDescription, VAR_OUT},
       {"NewLeaseDuration", VAR_PortMappingLeaseDuration, VAR_OUT},
       { 0 }
    }
};

static Action _AddPortMapping = {
    "AddPortMapping", AddPortMapping,
   (Param []) {
       {"NewRemoteHost", VAR_RemoteHost, VAR_IN},
       {"NewExternalPort", VAR_ExternalPort, VAR_IN},
       {"NewProtocol", VAR_PortMappingProtocol, VAR_IN},
       {"NewInternalPort", VAR_InternalPort, VAR_IN},
       {"NewInternalClient", VAR_InternalClient, VAR_IN},
       {"NewEnabled", VAR_PortMappingEnabled, VAR_IN},
       {"NewPortMappingDescription", VAR_PortMappingDescription, VAR_IN},
       {"NewLeaseDuration", VAR_PortMappingLeaseDuration, VAR_IN},
       { 0 }
    }
};

static Action _DeletePortMapping = {
    "DeletePortMapping", DeletePortMapping,
   (Param []) {
       {"NewRemoteHost", VAR_RemoteHost, VAR_IN},
       {"NewExternalPort", VAR_ExternalPort, VAR_IN},
       {"NewProtocol", VAR_PortMappingProtocol, VAR_IN},
       { 0 }
    }
};

static Action _GetExternalIPAddress = {
    "GetExternalIPAddress", GetExternalIPAddress,
   (Param []) {
       {"NewExternalIPAddress", VAR_ExternalIPAddress, VAR_OUT},
       { 0 }
    }
};



static PAction Actions[] = {
    &_SetConnectionType,
    &_GetConnectionTypeInfo,
    &_RequestConnection,
    &_ForceTermination,
    &_GetStatusInfo,
    &_GetNATRSIPStatus,
    &_GetGenericPortMappingEntry,
    &_GetSpecificPortMappingEntry,
    &_AddPortMapping,
    &_DeletePortMapping,
    &_GetExternalIPAddress,
    NULL
};


ServiceTemplate Template_WANPPPConnection = {
    "WANPPPConnection:1",
    WANPPPConnectionInit,	/* service initialization */
    WANPPPConnection_GetVar,	/* state variable handler */
    NULL,			/* xml generator */
    ARRAYSIZE(StateVariables)-1, StateVariables,
    Actions, 0,
    "urn:upnp-org:serviceId:WANPPPConn"
};

extern void enable_wan();


static int WANPPPConnectionInit(struct Service *psvc, service_state_t state)
{
    struct  itimerspec  timer;
    PWANPPPConnectionData pdata;
    PWANDevicePrivateData pdevdata;	
    char *wanif;

    /*
     * If the WAN interface is PPP then this should not be initialized.
     * I am a little paranoid so I'll check and return.
     */
    wanif = nvram_safe_get ( "wan_ifname" );
    if ( strstr ( wanif, "nas" ) ) {
        return 0;
    }


    switch (state) {
    case SERVICE_CREATE:
	pdevdata = (PWANDevicePrivateData) psvc->device->parent->opaque;
	pdata = (PWANPPPConnectionData) malloc(sizeof(WANPPPConnectionData));
	if (pdata) {
	    memset(pdata, 0, sizeof(WANPPPConnectionData));
	    pdata->connection_status = PPP_CONNECTING;
	    pdata->connected_time = time(NULL);
	    pdata->igd_generation = igd_config_generation;

	    mapmgr_update();
	    pdata->nportmappings = mapmgr_port_map_count();
	    
	    osl_ifaddr(pdevdata->ifname, &pdata->external_ipaddr);
	    psvc->opaque = (void *) pdata;
	    
	    /* once a second we want to update the statistics variables in the WANPPPConnection service */
	    memset(&timer, 0, sizeof(timer));
	    timer.it_interval.tv_sec = 30;
	    timer.it_value.tv_sec = 25;
	    pdata->eventhandle = enqueue_event(&timer, (event_callback_t)WANPPPConnection_Update, (void *) psvc );
	}
	break;

    case SERVICE_DESTROY:
	pdata = (PWANPPPConnectionData) psvc->opaque;
	
	timer_delete(pdata->eventhandle);
	free(pdata);
	break;
    } /* end switch */

    return 0;
}

int need_restart = 0;

void igd_restart_helper()
{
    if (need_restart) {
	need_restart = 0;
	osl_sys_restart();
    }
}

void igd_restart(int secs) 
{
    extern void delayed_call(uint seconds, voidfp_t f);

    need_restart = 1;
    delayed_call(secs, igd_restart_helper);
}

/* Get the NVRAM variable corresponding to the primary WAN interface. */
char *igd_pri_wan_var(char *prefix, int len, char *var)
{
	int unit;
	char tmp[100];

	for (unit = 0; unit < MAX_NVPARSE; unit ++) {
		snprintf(tmp, sizeof(tmp), "wan%d_primary", unit);
		if (nvram_match(tmp, "1")) {
			break;
                }
	}
	if (unit == MAX_NVPARSE)
		unit = 0;

	snprintf(prefix, len, "wan%d_%s", unit, var);

	return (prefix);
}

/* this is run once a second from WANPPPConnection_Update. */
static bool update_connection_status(PService psvc)
{
    PWANPPPConnectionData pdata = (PWANPPPConnectionData) psvc->opaque;
    PWANDevicePrivateData pdevdata = (PWANDevicePrivateData) psvc->device->parent->opaque;
    ppp_conn_t old_status = pdata->connection_status;
    bool wan_connected;

    wan_connected = osl_wan_isup(pdevdata->ifname);

    switch (pdata->connection_status) {
    case PPP_CONNECTING: 
	{
	    if (wan_connected) {
		pdata->connection_status = PPP_CONNECTED;
		pdata->connected_time = time(NULL);
	    } else if (pdata->connection_timeout-- <= 0) {
		pdata->connection_status = PPP_DISCONNECTED;
	    }
	}
	break;

    case PPP_CONNECTED: 
	{
	    if (!wan_connected) {
		pdata->connection_status = PPP_DISCONNECTED;
	    } 
	}
	break;
    case PPP_DISCONNECTING: 
	{
	    if (wan_connected) {
		pdata->connection_status = PPP_CONNECTING;
	    } else {
		pdata->connection_status = PPP_DISCONNECTED;
	    }
	}
	break;
	
    case PPP_DISCONNECTED: 
	{
	    if (wan_connected) {
		pdata->connection_status = PPP_CONNECTING;
	    }
	}
	break;
    default:
	UPNP_ERROR(("%s:%d Unexpected connection_status %d\n", __FILE__, __LINE__, pdata->connection_status));
    }

    if (old_status != pdata->connection_status) {
	mark_changed(psvc, VAR_ConnectionStatus);
    }

    return (old_status != pdata->connection_status);
}

/* this is run once a second from WANPPPConnection_Update. */
static bool update_external_address(PService psvc)
{
    PWANPPPConnectionData pdata = (PWANPPPConnectionData) psvc->opaque;
    PWANDevicePrivateData pdevdata = (PWANDevicePrivateData) psvc->device->parent->opaque;
    struct in_addr ipaddr = {0} ;

    if (osl_wan_isup(pdevdata->ifname)) {
	osl_ifaddr(pdevdata->ifname, &ipaddr);
    }
    if (pdata->external_ipaddr.s_addr != ipaddr.s_addr) {
	pdata->external_ipaddr = ipaddr;
	mark_changed(psvc, VAR_ExternalIPAddress);
    }

    return (pdata->external_ipaddr.s_addr != ipaddr.s_addr);
}


/* this is run once a second. */
static void WANPPPConnection_Update(timer_t t, PService psvc)
{
    PWANPPPConnectionData pdata = (PWANPPPConnectionData) psvc->opaque;

    if (igd_config_generation != pdata->igd_generation) {
	pdata->igd_generation = igd_config_generation;
	mapmgr_update();
	if (pdata->nportmappings != mapmgr_port_map_count()) {
	    pdata->nportmappings = mapmgr_port_map_count();
	    mark_changed(psvc, VAR_PortMappingNumberOfEntries);
	}
    }

    update_external_address(psvc);
    update_connection_status(psvc);

    if ((psvc->flags & VAR_CHANGED) == VAR_CHANGED) {
	update_all_subscriptions(psvc);
    }
}


static int WANPPPConnection_GetVar(struct Service *psvc, int varindex)
{
    PWANPPPConnectionData data = psvc->opaque;
    struct StateVar *var;
    time_t now;
    char **p, *cur;
    int len;
    ulong upstr;
    ulong dnstr;

    var = &(psvc->vars[varindex]);

    switch (varindex) {
    case VAR_PossibleConnectionTypes:
	cur = var->value;
	len = sizeof(var->value);
	for (p = PossibleConnectionTypes_allowedValueList; *p; p++) {
	    if (cur != var->value)
		cur = safe_snprintf(cur, &len, ",");
	    cur = safe_snprintf(cur, &len, "%s", *p);
	}
	break;

    case VAR_Uptime:
	if (data->connection_status != PPP_CONNECTED) {
	    sprintf(var->value, "%d", 0);
	} else {
	    now = time(NULL);
	    sprintf(var->value, "%u",
                    (u_int32) now - (u_int32) data->connected_time);
	}
	break;

    case VAR_ConnectionStatus:
	sprintf(var->value, "%s",
                ConnectionStatus_allowedValueList[data->connection_status]);
	break;

    case VAR_UpstreamMaxBitRate:
	if ( osl_max_bitrates ( "/dev/adsl0", &dnstr, &upstr ) ) {
	    sprintf(var->value, "%lu", upstr);
        }
	break;

    case VAR_DownstreamMaxBitRate:
	if ( osl_max_bitrates ( "/dev/adsl0", &dnstr, &upstr ) ) {
	    sprintf(var->value, "%lu", dnstr);
        }
	break;

    case VAR_PortMappingNumberOfEntries:
	sprintf(var->value, "%d", data->nportmappings);
	break;

    case VAR_ExternalIPAddress: {
	    const unsigned char *bytep;

	    bytep = (const unsigned char *) &(data->external_ipaddr.s_addr);
	    sprintf(var->value, "%d.%d.%d.%d", bytep[0], bytep[1], bytep[2],
                    bytep[3]);
        }
	break;

    case VAR_RSIPAvailable:
	sprintf(var->value, "%d", 1);
	break;

    case VAR_NATEnabled:
	sprintf(var->value, "%d", 1);
	break;
    } /* end-switch */

    return TRUE;
}


static int ForceTermination(UFILE *uclient, PService psvc, PAction ac,
                            pvar_entry_t args, int nargs)
{
    uint success = TRUE; /* assume no error will occur */
    PWANPPPConnectionData pdata = (PWANPPPConnectionData) psvc->opaque;
    char wanproto[100];
    char *wanif;

    /*
     * Our ConnectionType is always IP_Routed, so I don't need to check
     * that here.
     */
    if (pdata->connection_status == PPP_DISCONNECTED) {
	soap_error(uclient, SOAP_CONNECTIONALREADYTERMNATED);
	success = FALSE;
    } else if (pdata->connection_status != PPP_CONNECTED 
	       && pdata->connection_status != PPP_CONNECTING) {
	soap_error(uclient, SOAP_DISCONNECTINPROGRESS);
	success = FALSE;
    } else { 
        wanif = nvram_safe_get ( "wan_ifname" );
	if ( strcasecmp(wanif, "ppp_") == 0 )  {
            osl_igd_ppp_conn_disable ( wanif );
        } else {
	    igd_pri_wan_var(wanproto, sizeof(wanproto), "proto");
	    /* Save the wan_proto into NVRAM to restore when the igd is enabled */
	    nvram_set("upnp_wan_proto", nvram_safe_get(wanproto));
	    nvram_set(wanproto, "disabled");
	    nvram_commit();
            osl_igd_disable ();
        }
    }
    
    return success;
}

static int RequestConnection(UFILE *uclient, PService psvc, PAction ac,
                             pvar_entry_t args, int nargs)
{
    uint success = TRUE; /* assume no error will occur */
    PWANPPPConnectionData pdata = psvc->opaque;
    PWANDevicePrivateData pdevdata =
                   (PWANDevicePrivateData) psvc->device->parent->opaque;
    char wanproto[100];
    char *wanif;

    if (pdata->connection_status == PPP_UNCONFIGURED) {
	soap_error(uclient, SOAP_CONNECTIONNOTCONFIGURED);
	success = FALSE;
    } else if (pdata->connection_status == PPP_DISCONNECTED) {
	igd_pri_wan_var(wanproto, sizeof(wanproto), "proto");
        wanif = nvram_safe_get ( "wan_ifname" );
	if ( strcasecmp(wanif, "ppp_") == 0 ) {
            osl_igd_ppp_conn_enable ( wanif );
	    pdata->connection_status = PPP_CONNECTED;
        } else {
            if ( osl_link_status ( "/dev/bcmadsl0" ) == OSL_LINK_UP ) {
	        pdata->connection_status = PPP_CONNECTED;
                nvram_set ( pdevdata->ifname, "UP" );
            } else {
                osl_igd_enable ();
	        pdata->connection_status = PPP_CONNECTING;
                nvram_set ( pdevdata->ifname, "DOWN" );
            }
        }
        nvram_set ( wanproto, "enabled" );
    }
    nvram_commit();
    mark_changed(psvc, VAR_ConnectionStatus);

    return success;
}


static int SetConnectionType(UFILE *uclient, PService psvc, PAction ac,
                             pvar_entry_t args, int nargs)
{
    snprintf(psvc->vars[VAR_ConnectionType].value,
             sizeof(psvc->vars[VAR_ConnectionType].value), ac->params[0].value);

    return TRUE;
}
